library feedify;
import'dart:io' show Platform;
import 'package:feedify/logs.dart';
import 'package:feedify/src/manager/feed_api_manager.dart';
import 'package:feedify/src/manager/notification_manager.dart';
import 'package:feedify/src/model/model_firebase_app.dart';
import 'package:feedify/src/model/notification/model_notification.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';

import 'src/storage/app-prefs.dart';

/// entry point for SDK, this class contains all the configuration related methods that needs
/// to be used at runtime
Future<void> _firebaseMessagingBackgroundHandler(RemoteMessage message) async {
  _handleNotification(message);
}

class FeedSDK {
  static String feedUser = '';
  static String feedKey = '';
  static String feedDomain = '';
  static String notificationIco = '';
  static String startScreen = '';
  static final FeedSDK _singleton = FeedSDK._internal();
  AppPreferences? _appPreferences;

  FeedSDK._internal();

  _FeedSDK() {}

  /// used to initialise the SDK and Firebase
  ///
  /// [feedifyUser] username provided from Feedify console,
  /// [feedKey] key provided from Feedify console and
  /// [feedDomain] domain provided from Feedify console
  /// all the paramters are mandatory, if not provided then [UnimplementedError] is thrown
  static FeedSDK getInstance(
      {required feedifyUser, required feedifyKey, required feedifyDomain}) {
    if (feedifyUser.isEmpty || feedifyDomain.isEmpty || feedifyKey.isEmpty) {
      throw UnimplementedError('kindly provide valid values');
    }
    feedKey = feedifyKey;
    feedDomain = feedifyDomain;
    feedUser = feedifyUser;
    return _singleton;
  }

  /// icon to be used on the notification
  ///
  ///  [notificationIcon] - drawable icon to be used as icon to be shown inside notifications
  /// like: 'resource://drawable/ic_notification'
  static void setNotificationIcon(String notificationIcon) {
    notificationIco = notificationIcon;
  }

  /// sets the screen that needs to be started on click of the notification, and when set, passes all
  /// the notification related data like payload, custom params, to the passed screen, inside arguments
  ///
  ///  [screenNamed] - screen if set then all the notification related data will
  ///                      be passed to the screen when notification is clicked
  static void setStartScreen(String screenNamed) {
    startScreen = screenNamed;
  }

  ///used to initialise the SDK and Firebase
  ///- Note: All the required paramters can be found in google-services.json file downloaded from Firebase console
  ///except [appName] which can be any String

  Future<void> init(
      {required String appName,
      required String projectId,
      required String firebaseCurrentApiKey,
      required String firebaseMobileSdkAppId,
      required String firebaseStorageBucket,
      String? firebaseUrl,
      required String firebaseProjectNumber}) async {
    Logs.d('started init');
    await _initialiseApp(_getFirebaseModel(
        appName,
        projectId,
        firebaseCurrentApiKey,
        firebaseMobileSdkAppId,
        firebaseStorageBucket,
        firebaseUrl,
        firebaseProjectNumber));
    Logs.d('ended init');
    String? token = await FirebaseMessaging.instance.getToken();
    Logs.d('token :: $token');
    bool? saved = await saveToken(token);
    Logs.d(saved == true ? 'token saved' : 'token failed to save');
    FeedApiManager.getInstance().register(_appPreferences);
    _addMessageListener();
    _initialiseNotifications();
    if (Platform.isIOS) {
      _requestPermissionForIos();
    }
  }

  _initialiseApp(ModelFirebaseApp modelFirebaseApp) async {
    await Firebase.initializeApp(
        name: modelFirebaseApp.appName,
        options: FirebaseOptions(
            projectId: modelFirebaseApp.projectId,
            apiKey: modelFirebaseApp.firebaseCurrentApiKey,
            appId: modelFirebaseApp.firebaseMobileSdkAppId,
            databaseURL: modelFirebaseApp.firebaseUrl,
            messagingSenderId: modelFirebaseApp.firebaseProjectNumber,
            storageBucket: modelFirebaseApp.firebaseStorageBucket));
    var secondaryApp = Firebase.app(modelFirebaseApp.appName);
    Logs.d('Initialized :: $secondaryApp');
  }

  /**
   * _requestPermissionForIos is requried for Firebase
   * Permission is compulsory 
   */

  _requestPermissionForIos() async {
    FirebaseMessaging messaging = FirebaseMessaging.instance;
    NotificationSettings settings = await messaging.requestPermission(
      alert: true,
      announcement: false,
      badge: true,
      carPlay: false,
      criticalAlert: false,
      provisional: false,
      sound: true,
    );
  }

  ///provides Firebase device token unique for every device from preferences
  Future<String?> getToken() async {
    _appPreferences ??= AppPreferences();
    return await _appPreferences?.getString(AppPreferences.KEY_FCM);
  }

  ///saves Firebase device token [fcmToken] unique for every device

  Future<bool?> saveToken(String? fcmToken) async {
    _appPreferences ??= AppPreferences();
    return await _appPreferences?.saveString(AppPreferences.KEY_FCM, fcmToken);
  }

  ModelFirebaseApp _getFirebaseModel(
      String appName,
      String projectId,
      String firebaseCurrentApiKey,
      String firebaseMobileSdkAppId,
      String firebaseStorageBucket,
      String? firebaseUrl,
      String firebaseProjectNumber) {
    return ModelFirebaseApp(
        appName,
        projectId,
        firebaseCurrentApiKey,
        firebaseMobileSdkAppId,
        firebaseStorageBucket,
        firebaseUrl,
        firebaseProjectNumber);
  }

  void _addMessageListener() {
    FirebaseMessaging.instance.onTokenRefresh.listen((event) {
      saveToken(event);
    });

    FirebaseMessaging.onMessage.listen((event) {
      Logs.d('onMessage received');
      _handleNotification(event);
    });

    FirebaseMessaging.onMessageOpenedApp.listen((event) {
      Logs.d('onMessageOpenedApp');
    });

    FirebaseMessaging.onBackgroundMessage(_firebaseMessagingBackgroundHandler);
  }

  void _initialiseNotifications() {
    NotificationManager.getInstance().init();
  }
}

ModelNotification _getModelNotification(RemoteMessage event) {
  return ModelNotification.fromJson(event.data);
}

void _handleNotification(RemoteMessage event) {
  var notificationData = _getModelNotification(event);
  if (notificationData.type == 'ad') {
    _fetchAd(event.data);
  } else {
    // var notificationData = _getModelNotification(event);
    _showNotification(notificationData);
  }
}

void _showNotification(ModelNotification? notificationData) {
  if (notificationData != null) {
    NotificationManager.getInstance().showNotification(notificationData);
    String? id = notificationData.id;
    String? type = notificationData.type;

    if (id != null && id.isNotEmpty && type != null && type.isNotEmpty) {
      FeedApiManager.getInstance()
          .deliveryCount(notificationData.id!, notificationData.type!);
    }
  }
}

void _fetchAd(Map<String, dynamic> notificationData) async {
  var response = await FeedApiManager.getInstance().fetchAd(notificationData);
  Logs.d('_fetchAd model :: ${response?.id}');
  _showNotification(response);
  Logs.d(response.toString());
}
